"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const Resources_1 = require("../../../Resources");
const Icon_1 = require("office-ui-fabric-react/lib/Icon");
const objectGet = require("object-get");
const Columns = [{
        key: "benefit.title",
        fieldName: "benefit.title",
        name: Resources_1.default.getResource("Lists_BenefitsAnalysis_Fields_Title_DisplayName"),
        minWidth: 100,
        maxWidth: 180,
        isMultiline: true,
    },
    {
        key: "benefit.responsible",
        fieldName: "benefit.responsible",
        name: Resources_1.default.getResource("SiteFields_GtGainsResponsible_DisplayName"),
        minWidth: 50,
        maxWidth: 180,
        isMultiline: true,
    },
    {
        key: "title",
        fieldName: "title",
        name: Resources_1.default.getResource("SiteFields_GtMeasureIndicatorLookup_DisplayName"),
        minWidth: 50,
        maxWidth: 180,
    },
    {
        key: "unit",
        fieldName: "unit",
        name: Resources_1.default.getResource("SiteFields_GtMeasurementUnit_DisplayName"),
        minWidth: 50,
        maxWidth: 80,
    },
    {
        key: "startValue",
        fieldName: "startValue",
        name: Resources_1.default.getResource("SiteFields_GtStartValue_DisplayName"),
        minWidth: 50,
        maxWidth: 80,
        data: { fieldNameDisplay: "startValueDisplay" },
    },
    {
        key: "desiredValue",
        fieldName: "desiredValue",
        name: Resources_1.default.getResource("SiteFields_GtDesiredValue_DisplayName"),
        minWidth: 50,
        maxWidth: 80,
        data: { fieldNameDisplay: "desiredValueDisplay" },
    },
    {
        key: "measurements[0].value",
        fieldName: "measurements[0].value",
        name: Resources_1.default.getResource("BenefitsOverview_LatestValue"),
        minWidth: 50,
        maxWidth: 80,
        data: { fieldNameDisplay: "measurements[0].valueDisplay" },
    },
    {
        key: "measurements[0].achievement",
        fieldName: "measurements[0].achievement",
        name: Resources_1.default.getResource("String_AchievementOfObjectives"),
        minWidth: 50,
        maxWidth: 80,
        data: {
            onCustomRender: (item) => {
                const colValue = objectGet(item, "measurements[0].achievementDisplay");
                const trendIconProps = objectGet(item, "measurements[0].trendIconProps");
                if (colValue) {
                    return (React.createElement("span", null,
                        React.createElement("span", { style: { display: "inline-block", width: 20 } }, trendIconProps && React.createElement(Icon_1.Icon, Object.assign({}, trendIconProps))),
                        React.createElement("span", null, colValue)));
                }
                return null;
            },
        },
    },
    {
        fieldName: "allMeasurements",
        key: "allMeasurements",
        name: "",
        minWidth: 50,
        maxWidth: 80,
        data: {
            onCustomRender: (item, column, callbacks) => {
                if (item.measurements.length > 0) {
                    return React.createElement("a", { href: "#", onClick: _ => callbacks[column.fieldName](item) }, Resources_1.default.getResource("BenefitsOverview_AllMeasurements"));
                }
                return null;
            },
        },
    },
].map(col => (Object.assign(Object.assign({}, col), { isResizable: true })));
/**
 * Get columns
 *
 * @param {boolean} showSiteTitleColumn Show site title column
 */
function GetColumns(showSiteTitleColumn) {
    let _columns = Columns;
    if (showSiteTitleColumn) {
        _columns.unshift({
            fieldName: "siteTitle",
            key: "siteTitle",
            name: Resources_1.default.getResource("String_Project"),
            minWidth: 100,
            maxWidth: 180,
            data: {
                onCustomRender: (item, column, callbacks) => {
                    const colValue = objectGet(item, column.fieldName);
                    return React.createElement("a", { href: "#", onClick: _ => callbacks[column.fieldName](item) }, colValue);
                },
            },
        });
    }
    return _columns;
}
exports.GetColumns = GetColumns;
