"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BenefitBase_1 = require("./BenefitBase");
const Util_1 = require("../../../Util");
class BenefitMeasurement extends BenefitBase_1.BenefitBase {
    /**
     * Creates a new instance of BenefitMeasurement
     *
     * @param {IBenefitsSearchResult} result Search result
     * @param {number} fractionDigits Fraction digits for valueDisplay
     */
    constructor(result, fractionDigits = 2) {
        super(result);
        this.date = new Date(result.GtMeasurementDateOWSDATE);
        this.dateDisplay = Util_1.formatDate(result.GtMeasurementDateOWSDATE, "LL");
        this.value = !isNaN(parseFloat(result.GtMeasurementValueOWSNMBR)) ? parseFloat(result.GtMeasurementValueOWSNMBR) : null;
        if (this.value !== null) {
            this.valueDisplay = this.value.toFixed(fractionDigits);
        }
        this.indicatorId = parseInt(result.GtMeasureIndicatorLookupId || result.RefinableString59, 10);
    }
    /**
     * Calculate achievement
     *
     * @param {BenefitMeasurementIndicator} indicator Indicator
     * @param {number} fractionDigits Fraction digits used for achievementDisplay
     */
    calculcateAchievement(indicator, fractionDigits = 2) {
        this.indicator = indicator;
        let achievement = (((this.value - this.indicator.startValue) / (this.indicator.desiredValue - this.indicator.startValue)) * 100);
        this.achievement = achievement;
        this.achievementDisplay = `${achievement.toFixed(fractionDigits)}%`;
        return this;
    }
    /**
     * Set trend icon props
     *
     * @param {BenefitMeasurement} prevMeasurement Previous measurement
     */
    setTrendIconProps(prevMeasurement) {
        let shouldIncrease = this.indicator.desiredValue > this.indicator.startValue;
        if (this.achievement >= 100) {
            this.trendIconProps = { iconName: "Trophy", style: { color: "gold" } };
            return this;
        }
        if (prevMeasurement && prevMeasurement.value !== this.value) {
            let hasIncreased = this.value > prevMeasurement.value;
            if (shouldIncrease && hasIncreased || !shouldIncrease && !hasIncreased) {
                this.trendIconProps = { iconName: "StockUp", style: { color: "green" } };
            }
            else {
                this.trendIconProps = { iconName: "StockDown", style: { color: "red" } };
            }
        }
        return this;
    }
}
exports.BenefitMeasurement = BenefitMeasurement;
