"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BenefitBase_1 = require("./BenefitBase");
class BenefitMeasurementIndicator extends BenefitBase_1.BenefitBase {
    /**
     * Creates a new instance of BenefitMeasurementIndicator
     *
     * @param {IBenefitsSearchResult} result Search result
     * @param {number} fractionDigits Fraction digits for valueDisplay
     */
    constructor(result, fractionDigits = 2) {
        super(result);
        this.indicator = result.GtMeasureIndicatorOWSTEXT;
        this.startValue = !isNaN(parseFloat(result.GtStartValueOWSNMBR)) ? parseFloat(result.GtStartValueOWSNMBR) : null;
        if (this.startValue !== null) {
            this.startValueDisplay = this.startValue.toFixed(fractionDigits);
        }
        this.desiredValue = !isNaN(parseFloat(result.GtDesiredValueOWSNMBR)) ? parseFloat(result.GtDesiredValueOWSNMBR) : null;
        if (this.desiredValue !== null) {
            this.desiredValueDisplay = this.desiredValue.toFixed(fractionDigits);
        }
        this.unit = result.GtMeasurementUnitOWSCHCS;
    }
    /**
     * Set measurements
     *
     * @param {BenefitMeasurement[]} measurements Measurements
     */
    setMeasurements(measurements) {
        let _measurements = measurements.filter(m => m.indicatorId === this.id && m.webId === this.webId);
        _measurements = _measurements.map((m, i) => m.calculcateAchievement(this));
        _measurements = _measurements.map((m, i) => m.setTrendIconProps(_measurements[i + 1]));
        this.measurements = _measurements;
        return this;
    }
    /**
     * Set benefit
     *
     * @param {Benefit} benefit Benefit
     */
    setBenefit(benefit) {
        this.benefit = benefit;
        return this;
    }
}
exports.BenefitMeasurementIndicator = BenefitMeasurementIndicator;
