"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
//#region Imports
const sp_1 = require("@pnp/sp");
const Resources_1 = require("../../../Resources");
const SearchService_1 = require("../../../Services/SearchService");
const Benefit_1 = require("./Benefit");
const BenefitMeasurement_1 = require("./BenefitMeasurement");
const BenefitMeasurementIndicator_1 = require("./BenefitMeasurementIndicator");
//#endregion
/**
 * Fetches data based on selected data source (List or Search)
 *
 * @param {string} queryTemplate Query template
 * @param {string} dataSourceName Data source name
 */
function fetchData(queryTemplate, dataSourceName) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            let searchSettings = {
                Querytext: "*",
                QueryTemplate: queryTemplate,
                RowLimit: 500,
                SelectProperties: [
                    "Path",
                    "Title",
                    "ListItemId",
                    "SiteTitle",
                    "WebId",
                    "ContentTypeID",
                    "GtDesiredValueOWSNMBR",
                    "GtMeasureIndicatorOWSTEXT",
                    "GtMeasurementUnitOWSCHCS",
                    "GtStartValueOWSNMBR",
                    "GtMeasurementValueOWSNMBR",
                    "GtMeasurementCommentOWSMTXT",
                    "GtMeasurementDateOWSDATE",
                    "GtGainsResponsibleOWSUSER",
                    "GtGainsTurnoverOWSMTXT",
                    "GtGainsTypeOWSCHCS",
                    "GtPrereqProfitAchievementOWSMTXT",
                    "GtRealizationTimeOWSDATE",
                    "GtGainLookupId",
                    "GtMeasureIndicatorLookupId",
                    "GtGainsResponsible",
                    "RefinableString58",
                    "RefinableString59",
                    "RefinableString70",
                ],
                TrimDuplicates: false,
            };
            if (dataSourceName) {
                const dataSourcesList = new sp_1.Site(_spPageContextInfo.siteAbsoluteUrl).rootWeb.lists.getByTitle(Resources_1.default.getResource("Lists_DataSources_Title"));
                const [dataSource] = yield dataSourcesList.items.select("GtDpSearchQuery").filter(`Title eq '${dataSourceName}'`).get();
                searchSettings.QueryTemplate = dataSource.GtDpSearchQuery;
            }
            try {
                const { items } = yield SearchService_1.default.search(searchSettings);
                const benefits = items
                    .filter(res => res.ContentTypeID.indexOf("0x0100B384774BA4EBB842A5E402EBF4707367") === 0)
                    .map(res => new Benefit_1.Benefit(res));
                const measurements = items
                    .filter(res => res.ContentTypeID.indexOf("0x01007A831AC68204F04AAA022CFF06C7BAA2") === 0)
                    .map(res => new BenefitMeasurement_1.BenefitMeasurement(res))
                    .sort((a, b) => b.date.getTime() - a.date.getTime());
                let indicators = [];
                items
                    .filter(res => res.ContentTypeID.indexOf("0x0100FF4E12223AF44F519AF40C441D05DED0") === 0)
                    .forEach(res => {
                    let _benefitLookupId = res.GtGainLookupId || res.RefinableString58;
                    if (_benefitLookupId) {
                        let _benfitIds = _benefitLookupId.split(";").map(str => parseInt(str, 10));
                        _benfitIds.forEach(_benfitId => {
                            let _indicator = new BenefitMeasurementIndicator_1.BenefitMeasurementIndicator(res);
                            let [_benefit] = benefits.filter(b => b.id === _benfitId && b.webId === _indicator.webId);
                            if (_benefit) {
                                _indicator = _indicator
                                    .setMeasurements(measurements)
                                    .setBenefit(_benefit);
                                indicators.push(_indicator);
                            }
                        });
                    }
                });
                return indicators;
            }
            catch (err) {
                throw err;
            }
        }
        catch (err) {
            throw err;
        }
    });
}
exports.fetchData = fetchData;
