"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const Util = require("../../../Util");
const Icon_1 = require("office-ui-fabric-react/lib/Icon");
const Tooltip_1 = require("office-ui-fabric-react/lib/Tooltip");
/**
 * Render item column
 *
 * @param {any} item The item
 * @param {number} _index Index
 * @param {IDynamicPortfolioColumnConfig} column Column
 * @param {IDynamicPortfolioConfiguration} configuration Configuration
 * @param {Function} titleOnClick Title column on click
 */
const DynamicPortfolioItemColumn = (item, _index, column, configuration, truncLength, titleOnClick) => {
    let colValue = item[column.key];
    if (colValue) {
        if (colValue !== "" && (column.key.indexOf("OWSNMBR") !== -1 || column.key.indexOf("OWSCURR") !== -1)) {
            colValue = parseInt(colValue, 10);
        }
        switch (column.key) {
            case "Title": {
                return (React.createElement("a", { href: item.Path, onClick: titleOnClick }, colValue && colValue !== "DispForm.aspx" ? colValue : item.SiteTitle));
            }
            case "Path":
            case "URL": {
                return React.createElement("a", { href: item.Path }, colValue);
            }
        }
        switch (column.render) {
            case "Date": {
                return (React.createElement("span", null, colValue ? Util.formatDate(colValue, "LL") : null));
            }
            case "Note": {
                if (!truncLength || truncLength === "0") {
                    return React.createElement("span", { dangerouslySetInnerHTML: { __html: colValue } });
                }
                else {
                    return (React.createElement(Tooltip_1.TooltipHost, { content: colValue },
                        React.createElement("span", null, colValue.slice(0, parseInt(truncLength, 10)) + "...")));
                }
            }
            case "Currency": {
                let currValue = Util.toCurrencyFormat(colValue);
                return React.createElement("span", { title: currValue }, currValue);
            }
            case "Status": {
                let fieldName = Util.cleanSearchPropName(column.fieldName);
                if (configuration.statusFields.hasOwnProperty(fieldName)) {
                    const [statusProperties] = configuration.statusFields[fieldName].statuses.filter(({ statusValue }) => colValue === statusValue);
                    if (statusProperties) {
                        const statusComment = item[`${fieldName}CommentOWSMTXT`] ? `${colValue}: ${item[`${fieldName}CommentOWSMTXT`]}` : colValue;
                        const TooltipWrapper = ({ condition, wrapper, children }) => condition ? wrapper(children) : children;
                        return (React.createElement(TooltipWrapper, { condition: statusComment, wrapper: (children) => (React.createElement(Tooltip_1.TooltipHost, { content: statusComment, calloutProps: { gapSpace: 0 } }, children)) },
                            React.createElement("span", null,
                                React.createElement(Icon_1.Icon, { iconName: statusProperties.statusIconName, style: { color: statusProperties.statusColor } }),
                                "  ",
                                colValue)));
                    }
                }
                return (React.createElement("span", null, colValue));
            }
            case "URL": {
                let urlVals = colValue.split(",");
                return (React.createElement("a", { title: urlVals[1], href: urlVals[0] }, urlVals[1]));
            }
            case "Default": {
                return (React.createElement("span", { title: colValue }, colValue));
            }
            default: {
                return (React.createElement("span", { title: colValue }, colValue));
            }
        }
    }
    else {
        return null;
    }
};
exports.default = DynamicPortfolioItemColumn;
