"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Helpers_1 = require("../../Helpers");
const SearchService_1 = require("../../Services/SearchService");
/**
 * Transform result based on type
 *
 * @param {TypedHash<any>} res Result
 * @param {IDynamicPortfolioColumnConfig[]} columns Column configuration
 */
function transformResult(res, columns) {
    let result = Object.keys(res).reduce((obj, key) => {
        let value = res[key];
        let [col] = columns.filter(c => c.fieldName === key);
        if (col) {
            switch (col.render) {
                case "Date":
                    {
                        obj[key] = value && new Date(value);
                    }
                    break;
                default: {
                    obj[key] = value;
                }
            }
        }
        else {
            obj[key] = value;
        }
        return obj;
    }, Object.assign({}, res));
    result.Title = res.Title === "DispForm.aspx" ? res.SiteTitle : res.Title;
    result.Path = res.Path.indexOf("/Lists/Properties/") > -1 ? res.Path.split("/Lists")[0] : res.Path;
    return result;
}
exports.transformResult = transformResult;
/**
 * Query the REST Search API using @pnp/sp
 *
 * @param {string} queryText Query text
 * @param {IDynamicPortfolioViewConfig} viewConfig View configuration
 * @param {IDynamicPortfolioConfiguration} configuration DynamicPortfolioConfiguration
 */
function queryProjects(queryText, viewConfig, configuration) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const query = {
                Querytext: "*",
                SelectProperties: ["SiteTitle", ...configuration.columns.map(f => f.fieldName)],
                Refiners: configuration.refiners.map(ref => ref.key).join(","),
                QueryTemplate: [queryText, viewConfig.queryTemplate].filter(q => q).join(" "),
                RowLimit: 500,
                TrimDuplicates: false,
            };
            const { items, RawSearchResults } = yield SearchService_1.default.search(query);
            let refiners = Helpers_1.getObjectValue(RawSearchResults, "PrimaryQueryResult.RefinementResults.Refiners.results", []);
            return {
                primarySearchResults: items.map(res => transformResult(res, configuration.columns)),
                refiners,
            };
        }
        catch (err) {
            throw err;
        }
    });
}
exports.queryProjects = queryProjects;
