"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sp_1 = require("@pnp/sp");
const Resources_1 = require("../../Resources");
const SearchService_1 = require("../../Services/SearchService");
/**
 * Query the REST Search API using @pnp/sp. Find all project content types in the specified data source
 *
 * @param {string} dataSourceName Data source name
 * @param {number} rowLimit Row limit
 * @param {Array<string>} selectProperties Select properties
 */
function queryProjects(dataSourceName, rowLimit, selectProperties = []) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const dataSourcesList = new sp_1.Site(_spPageContextInfo.siteAbsoluteUrl).rootWeb.lists.getByTitle(Resources_1.default.getResource("Lists_DataSources_Title"));
            const [dataSource] = yield dataSourcesList.items.filter(`Title eq '${dataSourceName}'`).get();
            if (dataSource) {
                const { items } = yield SearchService_1.default.search({
                    Querytext: "*",
                    QueryTemplate: dataSource.GtDpSearchQuery,
                    SelectProperties: ["Path", "SiteTitle", "RefinableString52", "RefinableString53", "RefinableString54", "GtProjectManagerOWSUSER", "GtProjectOwnerOWSUSER", "LastModifiedTime", ...selectProperties],
                    RowLimit: rowLimit,
                    TrimDuplicates: false,
                });
                return items;
            }
            else {
                return [];
            }
        }
        catch (err) {
            throw err;
        }
    });
}
exports.queryProjects = queryProjects;
/**
 * Query the REST Search API using @pnp/sp. Find all webs in the specified data source
 *
 * @param {string} dataSourceName Data source name
 * @param {number} rowLimit Row limit
 */
function queryProjectWebs(dataSourceName, rowLimit) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const dataSourcesList = new sp_1.Site(_spPageContextInfo.siteAbsoluteUrl).rootWeb.lists.getByTitle(Resources_1.default.getResource("Lists_DataSources_Title"));
            const [dataSource] = yield dataSourcesList.items.filter(`Title eq '${dataSourceName}'`).get();
            if (dataSource) {
                let pathMatches = dataSource.GtDpSearchQuery.toString().split(" ").filter((substr) => { return substr.toLowerCase().indexOf("path:") === 0; });
                const dataSourceWithWebs = `contentclass:STS_Web ${pathMatches.join(" ")}`;
                const { items } = yield SearchService_1.default.search({
                    Querytext: "*",
                    QueryTemplate: dataSourceWithWebs,
                    SelectProperties: ["Title", "Path", "SiteLogo", "ViewsLifeTime"],
                    RowLimit: rowLimit,
                    TrimDuplicates: false,
                });
                return items;
            }
            else {
                return [];
            }
        }
        catch (err) {
            throw err;
        }
    });
}
exports.queryProjectWebs = queryProjectWebs;
