import * as React from "react";
import IResourceAllocationProps from "./IResourceAllocationProps";
import IResourceAllocationState from "./IResourceAllocationState";
/**
 * @class ResourceAllocation
 */
export default class ResourceAllocation extends React.Component<IResourceAllocationProps, IResourceAllocationState> {
    static displayName: string;
    static defaultProps: Partial<IResourceAllocationProps>;
    /**
     * Constructor
     *
     * @param {IResourceAllocationProps} props Props
     */
    constructor(props: IResourceAllocationProps);
    /**
     * Component did mount
     *
     * Fetching required data, and updating state
     */
    componentDidMount(): Promise<void>;
    /**
     * Renders the <ResourceAllocation /> component
     */
    render(): JSX.Element;
    /**
     * On filter change
     *
     * @param {IColumn} column Column
     * @param {IFilterItemProps[]} selectedItems Selected items
     */
    private onFilterChange;
    /**
     * Get filter items for column
     *
     * @param {IColumn} column Column
     */
    private getFilterItems;
    /**
     * Get filters
     */
    private getFilters;
    /**
    * Get filtered allocations
    *
    * @param {ProjectResourceAllocation[]} allocations Allocations
    * @param {Object} activeFilters Active filters
    */
    private getFilteredAllocations;
    /**
     * Get data for the timeline
     */
    private getTimelineData;
    /**
     * Timeline item renderer
     */
    private timelineItemRenderer;
    /**
     * On timeline item click, sets {allocationDisplay} in component state
     *
     * @param {React.MouseEvent} event Event
     * @param {any} item Item
     */
    private onTimelineItemClick;
    /**
     * On dismiss ResourceAllocationDetailsModal, sets {allocationDisplay} to null in component state
     */
    private onResourceAllocationDetailsModalDismiss;
    /**
     * Fetch data, parses the data, and creates arrays for [users] and [allocations]
     */
    private fetchData;
    /**
     * Searches for allocation items
     */
    private searchAllocationItems;
    /**
     * Fetches availability items from list on root
     */
    private fetchAvailabilityItems;
}
export { IResourceAllocationProps, IResourceAllocationState };
