"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
//#region Imports
const sp_1 = require("@pnp/sp");
const moment = require("moment");
const MessageBar_1 = require("office-ui-fabric-react/lib/MessageBar");
const Spinner_1 = require("office-ui-fabric-react/lib/Spinner");
const Utilities_1 = require("office-ui-fabric-react/lib/Utilities");
const React = require("react");
const react_calendar_timeline_1 = require("react-calendar-timeline");
const Helpers_1 = require("../../Helpers");
const Resources_1 = require("../../Resources");
const DataSourceService_1 = require("../../Services/DataSourceService");
const SearchService_1 = require("../../Services/SearchService");
const HeaderLabelFormats_1 = require("./HeaderLabelFormats");
const IResourceAllocationProps_1 = require("./IResourceAllocationProps");
const ResourceAllocationCommandBar_1 = require("./ResourceAllocationCommandBar");
const ResourceAllocationDetailsModal_1 = require("./ResourceAllocationDetailsModal");
const ResourceAllocationModels_1 = require("./ResourceAllocationModels");
const SubHeaderLabelFormats_1 = require("./SubHeaderLabelFormats");
//#endregion
/**
 * @class ResourceAllocation
 */
class ResourceAllocation extends React.Component {
    /**
     * Constructor
     *
     * @param {IResourceAllocationProps} props Props
     */
    constructor(props) {
        super(props);
        this.state = { isLoading: true, activeFilters: {} };
    }
    /**
     * Component did mount
     *
     * Fetching required data, and updating state
     */
    componentDidMount() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const data = yield this.fetchData();
                this.setState(Object.assign(Object.assign({}, data), { isLoading: false }));
            }
            catch (error) {
                this.setState({ error, isLoading: false });
            }
        });
    }
    /**
     * Renders the <ResourceAllocation /> component
     */
    render() {
        if (this.state.isLoading) {
            return (React.createElement(Spinner_1.Spinner, { type: Spinner_1.SpinnerType.large, label: Resources_1.default.getResource("ResourceAllocation_LoadingText") }));
        }
        if (this.state.error) {
            return React.createElement(MessageBar_1.MessageBar, null, Resources_1.default.getResource("ResourceAllocation_ErrorText"));
        }
        const data = this.getTimelineData(this.state);
        return (React.createElement(React.Fragment, null,
            React.createElement(MessageBar_1.MessageBar, null,
                React.createElement("div", { dangerouslySetInnerHTML: { __html: String.format(Resources_1.default.getResource("ResourceAllocation_LinkText"), `${_spPageContextInfo.siteAbsoluteUrl}/Lists/ResourceAllocation/AllItems.aspx?Source=${encodeURIComponent(window.location.href)}`) } })),
            React.createElement(ResourceAllocationCommandBar_1.default, { filters: this.getFilters(), onFilterChange: this.onFilterChange }),
            React.createElement(react_calendar_timeline_1.default, { groups: data.groups, items: data.items, itemRenderer: this.timelineItemRenderer, stickyHeader: true, stackItems: true, canMove: false, canChangeGroup: false, sidebarWidth: 220, headerLabelGroupHeight: 0, defaultTimeStart: moment().subtract(1, "months"), defaultTimeEnd: moment().add(1, "years"), headerLabelFormats: HeaderLabelFormats_1.HeaderLabelFormats, subHeaderLabelFormats: SubHeaderLabelFormats_1.SubHeaderLabelFormats },
                React.createElement(react_calendar_timeline_1.TimelineMarkers, null,
                    React.createElement(react_calendar_timeline_1.TodayMarker, null))),
            React.createElement(ResourceAllocationDetailsModal_1.default, { allocation: this.state.allocationDisplay, onDismiss: this.onResourceAllocationDetailsModalDismiss })));
    }
    /**
     * On filter change
     *
     * @param {IColumn} column Column
     * @param {IFilterItemProps[]} selectedItems Selected items
     */
    onFilterChange(column, selectedItems) {
        const { activeFilters } = Object.assign({}, this.state);
        if (selectedItems.length > 0) {
            activeFilters[column.fieldName] = selectedItems.map(i => i.value);
        }
        else {
            delete activeFilters[column.fieldName];
        }
        this.setState({ activeFilters });
    }
    /**
     * Get filter items for column
     *
     * @param {IColumn} column Column
     */
    getFilterItems(column) {
        let values = this.state.allocations.map(alloc => Helpers_1.getObjectValue(alloc, column.fieldName, ""));
        let valuesUnique = values.filter((value, index, self) => value && self.indexOf(value) === index);
        let valuesSorted = valuesUnique.sort((a, b) => a < b ? -1 : (a > b ? 1 : 0));
        return valuesSorted.map(value => ({ name: value, value }));
    }
    /**
     * Get filters
     */
    getFilters() {
        return this.props.filterColumns.map(column => ({ column, items: this.getFilterItems(column) }));
    }
    /**
    * Get filtered allocations
    *
    * @param {ProjectResourceAllocation[]} allocations Allocations
    * @param {Object} activeFilters Active filters
    */
    getFilteredAllocations(allocations, activeFilters) {
        allocations = Object.keys(activeFilters).reduce((_allocations, fieldName) => {
            return _allocations.filter(_alloc => {
                return activeFilters[fieldName].filter(_filterValue => {
                    return Helpers_1.getObjectValue(_alloc, fieldName, "").indexOf(_filterValue) !== -1;
                }).length > 0;
            });
        }, allocations);
        if (Object.keys(activeFilters).length === 1 && Object.keys(activeFilters)[0] === "user.name") {
            return allocations;
        }
        else {
            const users = allocations.map(alloc => alloc.name);
            const additionalAllocations = Object.keys(activeFilters).length > 0 ? this.state.portfolioAbsenceItems.filter(item => users.includes(item.name)) : [];
            return allocations.concat(additionalAllocations);
        }
    }
    /**
     * Get data for the timeline
     */
    getTimelineData({ users, allocations, activeFilters }) {
        allocations = this.getFilteredAllocations(allocations, activeFilters);
        const items = allocations.map((alloc, idx) => (Object.assign({ id: idx, title: alloc.toString(), group: alloc.user.id, type: alloc.type }, alloc)));
        const allocatedUsers = allocations.map(alloc => alloc.name);
        let groups = users.filter(user => allocatedUsers.includes(user.name)).map(user => ({ id: user.id, title: user.name }));
        groups = groups.sort((a, b) => (a.title < b.title) ? -1 : ((a.title > b.title) ? 1 : 0));
        return { groups, items };
    }
    /**
     * Timeline item renderer
     */
    timelineItemRenderer({ item, itemContext, getItemProps }) {
        const itemOpacity = item.allocationPercentage < 30 ? 0.3 : (item.allocationPercentage / 100);
        const itemColor = item.allocationPercentage < 40 ? "#000" : "#fff";
        const props = getItemProps({
            style: {
                color: itemColor,
                border: "none",
                cursor: "pointer",
                outline: "none",
            },
        });
        switch (item.type) {
            case ResourceAllocationModels_1.ProjectAllocationType.ProjectAllocation: {
                return (React.createElement("div", { key: props.key, className: props.className, style: Object.assign(Object.assign({}, props.style), { background: "rgb(51,153,51)", backgroundColor: `rgba(51,153,51,${itemOpacity})` }), title: itemContext.title, onClick: event => this.onTimelineItemClick(event, item) },
                    React.createElement("div", { className: "rct-item-content", style: { maxHeight: `${itemContext.dimensions.height}` } }, itemContext.title)));
            }
            case ResourceAllocationModels_1.ProjectAllocationType.Absence: {
                const isLeave = item.absence === Resources_1.default.getResource("Choice_GtResourceAbsence_Leave");
                const portfolioColorRGB = isLeave ? "205, 92, 92" : "26,111,179"; // Use red color if type=leave, else use blue portfolio color
                return (React.createElement("div", { key: props.key, className: props.className, style: Object.assign(Object.assign({}, props.style), { background: `rgb(${portfolioColorRGB})`, backgroundColor: `rgba(${portfolioColorRGB},${itemOpacity})` }), title: itemContext.title, onClick: event => this.onTimelineItemClick(event, item) },
                    React.createElement("div", { className: "rct-item-content", style: { maxHeight: `${itemContext.dimensions.height}` } }, itemContext.title)));
            }
        }
    }
    /**
     * On timeline item click, sets {allocationDisplay} in component state
     *
     * @param {React.MouseEvent} event Event
     * @param {any} item Item
     */
    onTimelineItemClick(event, item) {
        event.preventDefault();
        this.setState({ allocationDisplay: item });
    }
    /**
     * On dismiss ResourceAllocationDetailsModal, sets {allocationDisplay} to null in component state
     */
    onResourceAllocationDetailsModalDismiss() {
        this.setState({ allocationDisplay: null });
    }
    /**
     * Fetch data, parses the data, and creates arrays for [users] and [allocations]
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            const [searchResult, itemsAvailability] = yield Promise.all([
                this.searchAllocationItems(),
                this.fetchAvailabilityItems(),
            ]);
            const portfolioAbsenceItems = itemsAvailability.map(item => {
                const portfolioAbsence = new ResourceAllocationModels_1.ProjectResourceAllocation(item.GtResourceUser.Title, item.GtStartDate, item.GtEndDate, item.GtResourceLoad, ResourceAllocationModels_1.ProjectAllocationType.Absence, item.Title, item.GtResourceAbsenceComment);
                portfolioAbsence.absence = item.GtResourceAbsence;
                return portfolioAbsence;
            });
            const projectAllocations = searchResult.map(res => {
                const projectAllocation = new ResourceAllocationModels_1.ProjectResourceAllocation(res.RefinableString71, res.GtStartDateOWSDATE, res.GtEndDateOWSDATE, res.GtResourceLoadOWSNMBR, ResourceAllocationModels_1.ProjectAllocationType.ProjectAllocation, res.Title, res.GtResourceAbsenceCommentOWSTEXT);
                projectAllocation.project = { name: res.SiteTitle, url: res.SPWebUrl };
                projectAllocation.role = res.RefinableString72;
                return projectAllocation;
            });
            const allocations = [].concat(portfolioAbsenceItems, projectAllocations);
            let users = [];
            let userId = 0;
            for (let i = 0; i < allocations.length; i++) {
                let [user] = users.filter(r => r.name === allocations[i].name);
                if (!user) {
                    user = new ResourceAllocationModels_1.ProjectUser(userId, allocations[i].name);
                    users.push(user);
                    userId++;
                }
                allocations[i].user = user;
                user.allocations.push(allocations[i]);
            }
            return { users, allocations, portfolioAbsenceItems };
        });
    }
    /**
     * Searches for allocation items
     */
    searchAllocationItems() {
        return __awaiter(this, void 0, void 0, function* () {
            let queryTemplate;
            if (this.props.queryTemplate) {
                queryTemplate = this.props.queryTemplate;
            }
            else {
                queryTemplate = yield DataSourceService_1.default.getSourceByName(this.props.dataSourceName);
            }
            if (queryTemplate) {
                try {
                    const searchSettings = Object.assign({ QueryTemplate: queryTemplate }, this.props.searchConfiguration);
                    const { items } = yield SearchService_1.default.search(searchSettings);
                    return items;
                }
                catch (err) {
                    throw err;
                }
            }
            else {
                throw null;
            }
        });
    }
    /**
     * Fetches availability items from list on root
     */
    fetchAvailabilityItems() {
        return __awaiter(this, void 0, void 0, function* () {
            const web = new sp_1.Web(_spPageContextInfo.siteAbsoluteUrl);
            const itemsAvailabilityList = web.lists.getByTitle(Resources_1.default.getResource("Lists_ResourceAllocation_Title"));
            const itemsAvailability = yield itemsAvailabilityList
                .items
                .select("Title", "GtResourceUser/Title", "GtStartDate", "GtEndDate", "GtResourceLoad", "GtResourceAbsence")
                .expand("GtResourceUser")
                .get();
            return itemsAvailability;
        });
    }
}
ResourceAllocation.displayName = "ResourceAllocation";
ResourceAllocation.defaultProps = IResourceAllocationProps_1.ResourceAllocationDefaultProps;
__decorate([
    Utilities_1.autobind
], ResourceAllocation.prototype, "onFilterChange", null);
__decorate([
    Utilities_1.autobind
], ResourceAllocation.prototype, "timelineItemRenderer", null);
__decorate([
    Utilities_1.autobind
], ResourceAllocation.prototype, "onTimelineItemClick", null);
__decorate([
    Utilities_1.autobind
], ResourceAllocation.prototype, "onResourceAllocationDetailsModalDismiss", null);
exports.default = ResourceAllocation;
