"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
//#region Imports
const React = require("react");
const Resources_1 = require("../../../Resources");
const CommandBar_1 = require("office-ui-fabric-react/lib/CommandBar");
const ContextualMenu_1 = require("office-ui-fabric-react/lib/ContextualMenu");
const Utilities_1 = require("office-ui-fabric-react/lib/Utilities");
const FilterPanel_1 = require("../../@Components/FilterPanel");
const Helpers_1 = require("../../../Helpers");
const TaskOverviewDateIntervalPanel_1 = require("../TaskOverviewDateIntervalPanel");
//#endregion
class TasksOverviewCommandBar extends React.Component {
    /**
     * Constructor
     *
     * @param {ITasksOverviewCommandBarProps} props Props
     */
    constructor(props) {
        super(props);
        this.state = {};
    }
    /**
     * Get items
     */
    getItems() {
        const subItems = this.props.groupByOptions.map(({ fieldName, name }) => ({
            key: fieldName,
            fieldName,
            name,
            onClick: (event) => {
                event.preventDefault();
                this.props.onGroupByChanged({ fieldName, name });
            },
        }));
        return [
            {
                key: "Group",
                name: Helpers_1.getObjectValue(this.props, "groupBy.name", ""),
                iconProps: { iconName: "GroupedList" },
                itemType: ContextualMenu_1.ContextualMenuItemType.Header,
                onClick: event => event.preventDefault(),
                subMenuProps: { items: subItems },
            },
        ];
    }
    /**
     * Get far items
     */
    getFarItems() {
        return [
            {
                key: "DateInterval",
                name: Resources_1.default.getResource("TasksOverview_DateIntervalText"),
                iconProps: { iconName: "EventDate" },
                itemType: ContextualMenu_1.ContextualMenuItemType.Header,
                onClick: this.onOpenDateIntervalPanel,
            },
            {
                key: "Filter",
                name: null,
                iconOnly: true,
                iconProps: { iconName: "Filter" },
                itemType: ContextualMenu_1.ContextualMenuItemType.Header,
                onClick: this.onOpenFilerPanel,
            },
        ];
    }
    onOpenFilerPanel(event) {
        event.preventDefault();
        event.stopPropagation();
        this.setState((prevState) => ({ isFilterPanelOpen: !prevState.isFilterPanelOpen }));
    }
    onOpenDateIntervalPanel(event) {
        event.preventDefault();
        event.stopPropagation();
        this.setState((prevState) => ({ isDateIntervalPanelOpen: !prevState.isDateIntervalPanelOpen }));
    }
    onDismissFilterPanel() {
        this.setState({ isFilterPanelOpen: false });
    }
    onDismissDateIntervalPanel() {
        this.setState({ isDateIntervalPanelOpen: false });
    }
    /**
     * Renders the <TasksOverviewCommandBar /> component
     */
    render() {
        return (React.createElement("div", { style: { margin: "10px 0 10px 0" } },
            React.createElement(CommandBar_1.CommandBar, { items: this.getItems(), farItems: this.getFarItems() }),
            React.createElement(FilterPanel_1.default, { isOpen: this.state.isFilterPanelOpen, isLightDismiss: true, onDismiss: this.onDismissFilterPanel, filters: this.props.filters, onFilterChange: this.props.onFilterChange }),
            React.createElement(TaskOverviewDateIntervalPanel_1.default, { isOpen: this.state.isDateIntervalPanelOpen, onDismiss: this.onDismissDateIntervalPanel, headerText: Resources_1.default.getResource("TasksOverview_DateIntervalText"), defaultVisibleTimeStart: this.props.visibleTime.visibleTimeStart, defaultVisibleTimeEnd: this.props.visibleTime.visibleTimeEnd, onChange: this.props.onIntervalChange })));
    }
}
TasksOverviewCommandBar.displayName = "TasksOverviewCommandBar";
__decorate([
    Utilities_1.autobind
], TasksOverviewCommandBar.prototype, "onOpenFilerPanel", null);
__decorate([
    Utilities_1.autobind
], TasksOverviewCommandBar.prototype, "onOpenDateIntervalPanel", null);
__decorate([
    Utilities_1.autobind
], TasksOverviewCommandBar.prototype, "onDismissFilterPanel", null);
__decorate([
    Utilities_1.autobind
], TasksOverviewCommandBar.prototype, "onDismissDateIntervalPanel", null);
exports.default = TasksOverviewCommandBar;
