import * as React from "react";
import ITasksOverviewProps from "./ITasksOverviewProps";
import ITasksOverviewState from "./ITasksOverviewState";
/**
 * Component: TasksOverview
 */
export default class TasksOverview extends React.Component<ITasksOverviewProps, ITasksOverviewState> {
    static displayName: string;
    static defaultProps: Partial<ITasksOverviewProps>;
    private searchDelay;
    /**
     * Constructor
     *
     * @param {ITasksOverviewProps} props Props
     */
    constructor(props: ITasksOverviewProps);
    /**
     * Component did mount
     */
    componentDidMount(): Promise<void>;
    /**
     * Renders the <TasksOverview /> component
     */
    render(): JSX.Element;
    /**
   * Timeline item renderer
   */
    private timelineItemRenderer;
    /**
     * On dismiss <TasksOverviewDetailsModal />
     */
    private onTasksOverviewDetailsModalDismiss;
    /**
     * Get filter items for column
     *
     * @param {IColumn} column Column
     * @param {ITasksOverviewData} data Data
     */
    private getFilterItems;
    /**
     * Get filters
     *
     * @param {ITasksOverviewData} data Data
     */
    private getFilters;
    /**
     * On search
     *
     * @param {string} searchTerm Search term
     * @param {number} delayMs Delay in ms
     */
    private onSearch;
    /**
     * On filter change
     *
     * @param {IColumn} column Column
     * @param {IFilterItemProps[]} selectedItems Selected items
     */
    private onFilterChange;
    /**
    * On group by changed
    *
    * @param {Object} groupBy Group by
    */
    private onGroupByChanged;
    /**
    * On interval change
    *
    * @param {moment.Moment} visibleTimeStart Visible time start
    * @param {moment.Moment} visibleTimeEnd Visible time end
    */
    private onIntervalChange;
    /**
    * Get filtered tasks
    *
    * @param {TaskModel[]} tasks Tasks
    * @param {Object} activeFilters Active filters
    * @param {string} searchTerm Search term
    */
    private getFilteredTasks;
    /**
     * Sort items, either alphabetically or by a custom sort specified by [customSorts] in props
     *
     * @param {string[]} items Items
     * @param {string} fieldName Field name
     */
    private sortItems;
    /**
    * Get groups
    *
    * @param {string[]} items Items
    */
    private getGroups;
    /**
     * Get data
     *
     * @param {ITaskSearchResult[]} items Items
     */
    private getData;
    /**
     * Get data and visible dates for the timeline filtered by [activeFilters] and [searchTerm]
     *
     * @param {ITasksOverviewData} data Data
     */
    private getFilteredData;
    /**
     * On timeline item click, sets [selectedTask] in component [state]
     *
     * @param {TaskModel} task Task
     */
    private onTimelineItemClick;
    /**
     * Fetch items
     *
     * @param {string} dataSourceName Data source name
     * @param {SearchQuery} searchQuery Search confog
     */
    private fetchItems;
}
export { ITasksOverviewProps, ITasksOverviewState };
