"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
//#region Imports
const Resources_1 = require("../../Resources");
const React = require("react");
const moment = require("moment");
const logging_1 = require("@pnp/logging");
const react_calendar_timeline_1 = require("react-calendar-timeline");
const Spinner_1 = require("office-ui-fabric-react/lib/Spinner");
const MessageBar_1 = require("office-ui-fabric-react/lib/MessageBar");
const SearchBox_1 = require("office-ui-fabric-react/lib/SearchBox");
const ITasksOverviewProps_1 = require("./ITasksOverviewProps");
const TasksOverviewCommandBar_1 = require("./TasksOverviewCommandBar");
const TasksOverviewDetailsModal_1 = require("./TasksOverviewDetailsModal");
const SearchService_1 = require("../../Services/SearchService");
const DataSourceService_1 = require("../../Services/DataSourceService");
const TaskModel_1 = require("./TaskModel");
const Utilities_1 = require("office-ui-fabric-react/lib/Utilities");
const Helpers_1 = require("../../Helpers");
const TaskOverviewItem_1 = require("./TaskOverviewItem");
const HeaderLabelFormats_1 = require("./HeaderLabelFormats");
const SubHeaderLabelFormats_1 = require("./SubHeaderLabelFormats");
const TaskOverviewItemType_1 = require("./TaskOverviewItem/TaskOverviewItemType");
//#endregion
const LOG_TEMPLATE = "(TasksOverview) {0}: {1}";
/**
 * Component: TasksOverview
 */
class TasksOverview extends React.Component {
    /**
     * Constructor
     *
     * @param {ITasksOverviewProps} props Props
     */
    constructor(props) {
        super(props);
        this.state = {
            activeFilters: {},
            searchTerm: "",
            groupBy: props.groupByOptions[0],
            isLoading: true,
            itemType: TaskOverviewItemType_1.TaskOverviewItemType.Normal,
        };
    }
    /**
     * Component did mount
     */
    componentDidMount() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const items = yield this.fetchItems(this.props.dataSourceName, this.props.searchQuery);
                this.setState({ items, isLoading: false });
            }
            catch (err) {
                this.setState({ isLoading: false });
            }
        });
    }
    /**
     * Renders the <TasksOverview /> component
     */
    render() {
        const { isLoading, items, groupBy, itemType, selectedTask } = this.state;
        if (isLoading) {
            return React.createElement(Spinner_1.Spinner, { type: Spinner_1.SpinnerType.large, label: Resources_1.default.getResource("TasksOverview_LoadingText") });
        }
        if (!items) {
            return React.createElement(MessageBar_1.MessageBar, null, Resources_1.default.getResource("TasksOverview_ErrorText"));
        }
        const data = this.getData(items);
        const { groups, tasks, visibleTimeStart, visibleTimeEnd } = this.getFilteredData(data);
        return (React.createElement("div", null,
            React.createElement(MessageBar_1.MessageBar, null,
                React.createElement("div", { dangerouslySetInnerHTML: { __html: Resources_1.default.getResource("TasksOverview_InfoText") } })),
            React.createElement(TasksOverviewCommandBar_1.default, { visibleTime: { visibleTimeStart, visibleTimeEnd }, filters: this.getFilters(data), groupByOptions: this.props.groupByOptions, groupBy: groupBy, onFilterChange: this.onFilterChange, onGroupByChanged: this.onGroupByChanged, onIntervalChange: this.onIntervalChange }),
            React.createElement(SearchBox_1.SearchBox, { labelText: Resources_1.default.getResource("TasksOverview_SearchBoxPrompt"), onChanged: this.onSearch }),
            React.createElement("div", { style: { margin: "10px 0 0 0" } },
                React.createElement(react_calendar_timeline_1.default, { key: `Timeline_${itemType}`, groups: groups, items: tasks, stickyHeader: true, stackItems: true, canMove: false, canChangeGroup: false, sidebarWidth: 220, headerLabelGroupHeight: 0, lineHeight: itemType === TaskOverviewItemType_1.TaskOverviewItemType.DatesOnly ? 28 : 48, itemHeightRatio: 0.8, visibleTimeStart: visibleTimeStart.toDate().getTime(), visibleTimeEnd: visibleTimeEnd.toDate().getTime(), headerLabelFormats: HeaderLabelFormats_1.HeaderLabelFormats, subHeaderLabelFormats: SubHeaderLabelFormats_1.SubHeaderLabelFormats, itemRenderer: this.timelineItemRenderer },
                    React.createElement(react_calendar_timeline_1.TimelineMarkers, null,
                        React.createElement(react_calendar_timeline_1.TodayMarker, null)))),
            React.createElement(TasksOverviewDetailsModal_1.default, { task: selectedTask, onDismiss: this.onTasksOverviewDetailsModalDismiss })));
    }
    /**
   * Timeline item renderer
   */
    timelineItemRenderer({ item, itemContext, getItemProps }) {
        const itemProps = getItemProps(item.itemProps);
        return (React.createElement(TaskOverviewItem_1.default, { itemProps: itemProps, item: item, itemContext: itemContext, type: this.state.itemType, onItemClick: this.onTimelineItemClick }));
    }
    /**
     * On dismiss <TasksOverviewDetailsModal />
     */
    onTasksOverviewDetailsModalDismiss() {
        this.setState({ selectedTask: null });
    }
    /**
     * Get filter items for column
     *
     * @param {IColumn} column Column
     * @param {ITasksOverviewData} data Data
     */
    getFilterItems(column, data) {
        let values = [];
        data.tasks.forEach(task => {
            let value = Helpers_1.getObjectValue(task, `item.${column.fieldName}`, "").split(";");
            values.push(...value);
        });
        let valuesUnique = values.filter((value, index, self) => value && self.indexOf(value) === index);
        let valuesSorted = this.sortItems(valuesUnique, column.fieldName);
        return valuesSorted.map(value => ({ name: value, value }));
    }
    /**
     * Get filters
     *
     * @param {ITasksOverviewData} data Data
     */
    getFilters(data) {
        return this.props.filterColumns.map(column => {
            return { column, items: this.getFilterItems(column, data) };
        });
    }
    /**
     * On search
     *
     * @param {string} searchTerm Search term
     * @param {number} delayMs Delay in ms
     */
    onSearch(searchTerm, delayMs = 500) {
        searchTerm = searchTerm.toLowerCase();
        if (this.searchDelay) {
            clearTimeout(this.searchDelay);
        }
        this.searchDelay = window.setTimeout(() => {
            logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "onSearch", `Updating search term to ${searchTerm}`), level: 1 /* Info */ });
            this.setState({ searchTerm });
        }, delayMs);
    }
    /**
     * On filter change
     *
     * @param {IColumn} column Column
     * @param {IFilterItemProps[]} selectedItems Selected items
     */
    onFilterChange(column, selectedItems) {
        const { activeFilters } = Object.assign({}, this.state);
        if (selectedItems.length > 0) {
            activeFilters[column.fieldName] = selectedItems.map(i => i.value);
        }
        else {
            delete activeFilters[column.fieldName];
        }
        logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "onFilterChange", `Filter changed for ${column.fieldName}`), data: activeFilters, level: 1 /* Info */ });
        this.setState({ activeFilters });
    }
    /**
    * On group by changed
    *
    * @param {Object} groupBy Group by
    */
    onGroupByChanged(groupBy) {
        logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "onGroupByChanged", `Group by changed to ${groupBy.fieldName}`), level: 1 /* Info */ });
        const itemType = (groupBy && groupBy.fieldName === "Title") ? TaskOverviewItemType_1.TaskOverviewItemType.DatesOnly : TaskOverviewItemType_1.TaskOverviewItemType.Normal;
        this.setState({ groupBy, itemType });
    }
    /**
    * On interval change
    *
    * @param {moment.Moment} visibleTimeStart Visible time start
    * @param {moment.Moment} visibleTimeEnd Visible time end
    */
    onIntervalChange(visibleTimeStart, visibleTimeEnd) {
        this.setState({ visibleTimeStart, visibleTimeEnd });
    }
    /**
    * Get filtered tasks
    *
    * @param {TaskModel[]} tasks Tasks
    * @param {Object} activeFilters Active filters
    * @param {string} searchTerm Search term
    */
    getFilteredTasks(tasks, activeFilters, searchTerm) {
        logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "getFilteredTasks", "Get filtered tasks"), level: 1 /* Info */ });
        tasks = Object.keys(activeFilters).reduce((_tasks, fieldName) => {
            return _tasks.filter(_task => {
                return activeFilters[fieldName].filter(_filterValue => {
                    return Helpers_1.getObjectValue(_task, `item.${fieldName}`, "").indexOf(_filterValue) !== -1;
                }).length > 0;
            });
        }, tasks);
        tasks = tasks.filter(_task => _task.title.toLowerCase().indexOf(searchTerm) !== -1);
        return tasks;
    }
    /**
     * Sort items, either alphabetically or by a custom sort specified by [customSorts] in props
     *
     * @param {string[]} items Items
     * @param {string} fieldName Field name
     */
    sortItems(items, fieldName) {
        const customSort = Helpers_1.getObjectValue(this, `props.customSorts.${fieldName}`, null);
        if (customSort) {
            items = [...customSort, ""].filter(i => items.indexOf(i) !== -1);
        }
        else {
            items = items.sort((a, b) => a < b ? -1 : (a > b ? 1 : 0));
        }
        return items;
    }
    /**
    * Get groups
    *
    * @param {string[]} items Items
    */
    getGroups(items) {
        const groupByValues = items.map(task => Helpers_1.getObjectValue(task, this.state.groupBy.fieldName, ""));
        const groupByValuesUnique = groupByValues.filter((value, index, self) => self.indexOf(value) === index && value.indexOf(";") === -1);
        const groupByValuesSorted = this.sortItems(groupByValuesUnique, this.state.groupBy.fieldName);
        const groups = groupByValuesSorted.map((title, id) => ({ id, title }));
        return groups;
    }
    /**
     * Get data
     *
     * @param {ITaskSearchResult[]} items Items
     */
    getData(items) {
        logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "getData", "Getting data"), level: 1 /* Info */ });
        const groups = this.getGroups(items);
        let tasks = items.map((task, id) => {
            const [group] = groups.filter(grp => grp.title === Helpers_1.getObjectValue(task, this.state.groupBy.fieldName, ""));
            return group ? new TaskModel_1.TaskModel(id, group.id, task.Title, task.StartDateOWSDATE, task.DueDateOWSDATE, task) : null;
        });
        tasks = tasks.filter(task => task && task.start_time && task.end_time);
        return { groups, tasks };
    }
    /**
     * Get data and visible dates for the timeline filtered by [activeFilters] and [searchTerm]
     *
     * @param {ITasksOverviewData} data Data
     */
    getFilteredData(data) {
        logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "getFilteredData", "Getting filtered data"), level: 1 /* Info */ });
        let { activeFilters, searchTerm, visibleTimeStart, visibleTimeEnd } = Object.assign({}, this.state);
        let tasks = this.getFilteredTasks(data.tasks, activeFilters, searchTerm);
        let groups = data.groups.filter(grp => tasks.filter(item => item.group === grp.id).length > 0);
        visibleTimeStart = visibleTimeStart || tasks.map(t => t.start_time).reduce((a, b) => a < b ? a : b, moment()).startOf("month");
        visibleTimeEnd = visibleTimeEnd || tasks.map(t => t.end_time).reduce((a, b) => a > b ? a : b, moment()).endOf("month");
        return { groups, tasks, visibleTimeStart, visibleTimeEnd };
    }
    /**
     * On timeline item click, sets [selectedTask] in component [state]
     *
     * @param {TaskModel} task Task
     */
    onTimelineItemClick(task) {
        this.setState({ selectedTask: task });
    }
    /**
     * Fetch items
     *
     * @param {string} dataSourceName Data source name
     * @param {SearchQuery} searchQuery Search confog
     */
    fetchItems(dataSourceName, searchQuery) {
        return __awaiter(this, void 0, void 0, function* () {
            const queryTemplate = yield DataSourceService_1.default.getSourceByName(dataSourceName);
            if (queryTemplate) {
                try {
                    const _searchQuery = Object.assign({ QueryTemplate: queryTemplate }, searchQuery);
                    const { items } = yield SearchService_1.default.search(_searchQuery);
                    logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "fetchItems", `Successfully fetched ${items.length} items`), level: 1 /* Info */ });
                    return items;
                }
                catch (err) {
                    logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "fetchItems", `An error occured retrieving items.`), level: 2 /* Warning */ });
                    return null;
                }
            }
            else {
                logging_1.Logger.log({ message: String.format(LOG_TEMPLATE, "fetchItems", `Data source with name ${dataSourceName} not found.`), level: 2 /* Warning */ });
                return null;
            }
        });
    }
}
TasksOverview.displayName = "TasksOverview";
TasksOverview.defaultProps = ITasksOverviewProps_1.TasksOverviewDefaultProps;
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "timelineItemRenderer", null);
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "onTasksOverviewDetailsModalDismiss", null);
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "onSearch", null);
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "onFilterChange", null);
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "onGroupByChanged", null);
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "onIntervalChange", null);
__decorate([
    Utilities_1.autobind
], TasksOverview.prototype, "onTimelineItemClick", null);
exports.default = TasksOverview;
